% pac_compareLfoFreqs: Compares Resultant Vector Length, Modulation Index,
%                      and Mean Angle across LFO of 0.5, 1, 2, 4, and 8 Hz
%                      for each channel.
%
% Usage:
%   >>  [vectL, mAngle, modIdx] = pac_compareLfoFreqs(EEG, lfoFreqRange, ...
%                                   hfoFilter, hfoAmpPercentile, plotFlag);

% Author: Makoto Miyakoshi SCCN,INC,UCSD
% History
% 06/24/2013 ver 1.0 by Makoto. Created.

% Copyright (C) 2013, Makoto Miyakoshi SCCN,INC,UCSD
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function [vectL, mAngle, modIdx] = pac_compareLfoFreqs(EEG, lfoFreqRange, hfoFilter, hfoAmpPercentile, plotFlag)
%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% add folder to path %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~exist('circ_rtest','file')
    p = which('eegplugin_pac');
    p = p(1:findstr(p,'eegplugin_pac.m')-1);
    p = genpath(p);
    addpath(p);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% prepare input data %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%
tmpEEG = double(EEG.data);

%%%%%%%%%%%%%%%%%%%
%%% compute HFO %%%
%%%%%%%%%%%%%%%%%%%
% hiPassOrd = pop_firwsord('hamming', EEG.srate, 20);
% HFO = pop_firws(EEG, 'ftype', 'highpass', 'fcutoff', 100, 'wtype', 'hamming', 'forder', hiPassOrd);
% HFO = HFO.data;
HFO = filt_fir1fft(tmpEEG', 100, 0, EEG.srate);
analyAmp             = abs(hilbert(HFO))';
[analyAmpSort,index] = sort(analyAmp, 2, 'descend');
edge                 = ceil(hfoAmpPercentile/100*length(analyAmp));
edgeIdx              = index(:,1:edge);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% compute LFO and MI - Loop for LFO freqs, HFO percentiles, and Channels  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
numLoop = EEG.nbchan*(length(lfoFreqRange)-1);
counter   = 0;
h = waitbar(counter/numLoop, 'Now processing...');
vectL   = zeros(EEG.nbchan,length(lfoFreqRange)-1);
modIdx  = zeros(EEG.nbchan,length(lfoFreqRange)-1);

for n = 1:length(lfoFreqRange)-1
    tmpLoFreq   = lfoFreqRange(n);
    df          = tmpLoFreq*0.5;
%     bandPassOrd = pop_firwsord('hamming', EEG.srate, df)
%     LFO = pop_firws(EEG, 'ftype', 'bandpass', 'fcutoff', [lfoFreqRange(n) lfoFreqRange(n+1)], 'wtype', 'hamming', 'forder', bandPassOrd);
%     LFO = LFO.data;
    LFO = filt_fir1fft(tmpEEG', lfoFreqRange(n), lfoFreqRange(n+1), EEG.srate);
    analyPhase  = angle(hilbert(LFO))';
    
    for ch = 1:EEG.nbchan
        tmpChHfo    = analyAmp(  ch,edgeIdx(ch,:));
        tmpChPhase  = analyPhase(ch,edgeIdx(ch,:));
        vectL(ch,n) = circ_r(tmpChPhase');
        mAngle(ch,n) = circ_mean(tmpChPhase');
        modIdx(ch,n) = abs(mean(tmpChHfo.*exp(1i*tmpChPhase)));
        counter = counter+1;
        waitbar(counter/numLoop, h);
    end
end
close(h)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Resul Vect Len Stats %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
subplot(1,2,1)

[f,xi]=ksdensity(zscore(vectL(:)));

% histgram
hist(zscore(vectL(:)),40);
h = findobj(gca, 'Type', 'patch');
set(h, 'FaceColor', [0.93 0.96 1]);
hold on

% fitted distribution
[f,xi] = ksdensity(zscore(vectL(:)));
histData = hist(zscore(vectL(:)),40);
normF    = f*(max(histData)/max(f));
plot(xi,normF,'r','LineWidth',2);

% 1-5 percentiles
cutoff95  = ksdensity(zscore(vectL(:)), 0.95, 'function', 'icdf');
cutoff99  = ksdensity(zscore(vectL(:)), 0.99, 'function', 'icdf');
zRvl      = reshape(zscore(vectL(:)), size(vectL));
rvl95Mask = bsxfun(@ge, zRvl, cutoff95);
rvl99Mask = bsxfun(@ge, zRvl, cutoff99);
line([cutoff95 cutoff95],[0 max(histData)], 'Color', [0.7 0.7 0.7], 'LineWidth', 2)
line([cutoff99 cutoff99],[0 max(histData)], 'Color', [0.7 0.7 0.7], 'LineWidth', 2)
text(cutoff95, max(histData), '95%')
text(cutoff99, max(histData), '99%')

%%%%%%%%%%%%%%%%%%%%%
%%% Mod Ind Stats %%%
%%%%%%%%%%%%%%%%%%%%%
subplot(1,2,2)

[f,xi]=ksdensity(zscore(modIdx(:)));

% histgram
hist(zscore(modIdx(:)),40);
h = findobj(gca, 'Type', 'patch');
set(h, 'FaceColor', [0.93 1 0.96]);
hold on

% fitted distribution
[f,xi] = ksdensity(zscore(modIdx(:)));
histData = hist(zscore(modIdx(:)),40);
normF    = f*(max(histData)/max(f));
plot(xi,normF,'r','LineWidth',2);

% 1-5 percentiles
cutoff95 = ksdensity(zscore(modIdx(:)), 0.95, 'function', 'icdf');
cutoff99 = ksdensity(zscore(modIdx(:)), 0.99, 'function', 'icdf');
zMi      = reshape(zscore(modIdx(:)), size(modIdx));
mi95Mask = bsxfun(@ge, zMi, cutoff95);
mi99Mask = bsxfun(@ge, zMi, cutoff99);
line([cutoff95 cutoff95],[0 max(histData)], 'Color', [0.7 0.7 0.7], 'LineWidth', 2)
line([cutoff99 cutoff99],[0 max(histData)], 'Color', [0.7 0.7 0.7], 'LineWidth', 2)
text(cutoff95, max(histData), '95%')
text(cutoff99, max(histData), '99%')

% finalMask = rvl95Mask.*mi95Mask;
finalMask = rvl95Mask;

%%%%%%%%%%%%%%%%%%%%%
%%% Plot results  %%%
%%%%%%%%%%%%%%%%%%%%%
if plotFlag == 0
    return
end

figCounter  = 0;
plotCounter = 0;
if EEG.nbchan < 26
    subplotRow = ceil(EEG.nbchan/5);
else
    subplotRow = 6;
end

if EEG.nbchan < 5
    subplotColumn = EEG.nbchan;
else
    subplotColumn = 5;
end
    
for ch = 1:EEG.nbchan
    
    if figCounter   == 0 || plotCounter == 30;
        figCounter  = figCounter+1;
        plotCounter = 0;
        myfigure(EEG, figCounter, hfoFilter, hfoAmpPercentile);
    end
    
    % open subplot
    plotCounter = plotCounter+1;
    subplot(subplotRow,subplotColumn,plotCounter)
    
    % if 95 percentile
    if nnz(finalMask(ch,:))
        bar([0.5 1.5 2.5 3.5], finalMask(ch,:)*max(modIdx(:))*1.2, 1, 'EdgeColor', 'none')
        h = findobj(gca, 'Type', 'patch');
        set(h, 'FaceColor', [1 0.8 0.8]);
        hold on
    end
    
    % plot
    [ax,h1,h2] = plotyy(0.5:length(lfoFreqRange)-1, modIdx(ch,:), 0.5:length(lfoFreqRange)-1, vectL(ch,:));
    
    % set up Modulation Index
    xlim(ax(1), [0 length(lfoFreqRange)-1])
    ylim(ax(1), [0 max(modIdx(:))*1.2])
    set(ax(1), 'XTick', 0:length(lfoFreqRange)-1, 'XTickLabel', {'0.5' '1' '2' '4' '8'}, 'YTick', 0:10:200, 'YColor', 'k', 'Box', 'off')
    set(h1, 'Color', [0 0 0], 'LineStyle', ':', 'LineWidth', 2, 'Marker', '.', 'MarkerSize', 20)
    
    text(0.15,max(modIdx(:)*1.05),['Ch ' num2str(ch)],'FontSize', 16);
    
    % set up Resultant Vector Length
    xlim(ax(2), [0 length(lfoFreqRange)-1])
    ylim(ax(2), [0 1])
    set(ax(2), 'XTick', 0:length(lfoFreqRange)-1, 'XTickLabel', {'0.5' '1' '2' '4' '8'}, 'YTick', 0:0.2:1, 'YColor', [0 0 0.75], 'Box', 'off')
    set(h2, 'Color', [0 0 0.75], 'LineStyle', ':', 'LineWidth', 2, 'Marker', 'o', 'MarkerSize', 6)
    
    % only for the first plot
    if ch == 1
        set(get(ax(1), 'XLabel'), 'String', 'Frequency (Hz)', 'FontSize', 14)
        set(get(ax(1), 'YLabel'), 'String', 'Modulation Index', 'FontSize', 14)
        set(get(ax(2), 'YLabel'), 'String', 'Resul Vect Len', 'FontSize', 14)
    end
    
    %%%%%%%%%%%%%%%%%%%%%%
    %%% overplot polar %%%
    %%%%%%%%%%%%%%%%%%%%%%
    ax1Posi    = get(ax(2),'Position');
    % ax1OutPosi = get(ax(2),'OuterPosition');
    
    % determine child axis (circular plot) height and position
    childAxHeight = ax1Posi(4)*0.25;
    childAxBottom = ax1Posi(2) + ax1Posi(4)*0.85;
    if childAxBottom + childAxHeight > 1
        childAxHeight = childAxHeight-(childAxBottom+childAxHeight-1);
    end
    if childAxHeight < 0.06
        childAxHeight = 0.06;
        childAxBottom = ax1Posi(2) + ax1Posi(4)*0.7;
    end
    circPlotPosi = [ax1Posi(1) childAxBottom ax1Posi(3) childAxHeight];
    
    ax3 = axes('Position', circPlotPosi);
    colorCodes  = [0 0 0; 0.4 0.4 0.4; 0.6 0.6 0.6; 0.8 0.8 0.8;];
    h = plotSimplePolar([0 mAngle(ch,1)], [0 1]);
    set(h, 'LineWidth', 3, 'Color', colorCodes(1,:))
    hold on
    h = plotSimplePolar([0 mAngle(ch,2)], [0 1]);
    set(h, 'LineWidth', 3, 'Color', colorCodes(2,:))
    h = plotSimplePolar([0 mAngle(ch,3)], [0 1]);
    set(h, 'LineWidth', 3, 'Color', colorCodes(3,:))
    h = plotSimplePolar([0 mAngle(ch,4)], [0 1]);
    set(h, 'LineWidth', 3, 'Color', colorCodes(4,:))
    set(ax3, 'box', 'off')
end

% finalize the halfway figure
axcopy(gcf, 'if ~isempty(get(gca, ''''userdata'''')), eval(get(gca, ''''userdata'''')); end');


    
    
function myfigure(EEG, figCounter, hfoFilter, hfoAmpPercentile)
    
    % say good bye to the current figure
    axcopy(gcf, 'if ~isempty(get(gca, ''''userdata'''')), eval(get(gca, ''''userdata'''')); end');

    figure
    set(gcf, 'Name', 'pac_compareLfoFreqs()', 'NumberTitle','off')
    set(gcf,'Color', [0.93 0.96 1]);
    maxFigNum = ceil(EEG.nbchan/30);
    if hfoFilter(2) == 0
        hfoFilter(2) = round(EEG.srate/2);
    end
    str = sprintf('LFO freq comparison (HFO %3.0f-%3.0fHz and %3.1f percentile amplitude)\nIn the polar plot, black, dark gray, midle gray, light gray represents 0.5-1, 1-2, 2-4, and 4-8Hz.',...
        hfoFilter(1), hfoFilter(2), hfoAmpPercentile);
    annotation(gcf,'textbox', [0 1 1 0],'String', str,...
        'HorizontalAlignment','center', 'FontSize',16,...
        'FitBoxToText','off', 'LineStyle','none');   
    